@@ -25,23 +25,64 @@ The main module for accessing a vSphere server.
 # under the License.
 
 
+from __future__ import absolute_import, division, print_function
+
 import logging
 import os
-import suds
+import socket
 import time
 
-from urllib2 import URLError
+import suds
+from six.moves import http_client
+from six.moves.urllib.error import URLError
+from six.moves.urllib.request import HTTPSHandler, build_opener
 from suds.plugin import MessagePlugin
-from suds.transport import TransportError
+from suds.transport.http import HttpTransport, TransportError
 
-from psphere import soap, ManagedObject
+from psphere import ManagedObject, soap
 from psphere.config import _config_value
-from psphere.errors import (ConfigError, ObjectNotFoundError, TaskFailedError,
-                            NotLoggedInError)
+from psphere.errors import (
+    ConfigError, NotLoggedInError, ObjectNotFoundError, TaskFailedError,
+)
 from psphere.managedobjects import ServiceInstance, Task, classmapper
 
 logger = logging.getLogger(__name__)
 
+
+class HTTPSClientAuthHandler(HTTPSHandler):
+    def __init__(self, context):
+        HTTPSHandler.__init__(self)
+        self.context = context
+
+    def https_open(self, req):
+        return self.do_open(self.getConnection, req)
+
+    def getConnection(self, host, timeout=300):
+        return http_client.HTTPSConnection(host, context=self.context)
+
+
+class HTTPSClientContextTransport(HttpTransport):
+    def __init__(self, context, *args, **kwargs):
+        HttpTransport.__init__(self, *args, **kwargs)
+        self.context = context
+
+    def u2open(self, u2request):
+        """
+        Open a connection.
+        @param u2request: A urllib2 request.
+        @type u2request: urllib2.Requet.
+        @return: The opened file-like urllib2 object.
+        @rtype: fp
+        """
+        tm = self.options.timeout
+        url = build_opener(HTTPSClientAuthHandler(self.context))
+        if self.u2ver() < 2.6:
+            socket.setdefaulttimeout(tm)
+            return url.open(u2request)
+        else:
+            return url.open(u2request, timeout=tm)
+
+
 class Client(suds.client.Client):
     """A client for communicating with a VirtualCenter/ESX/ESXi server
 
@@ -63,7 +104,7 @@ class Client(suds.client.Client):
     :type plugins: list of classes
     """
     def __init__(self, server=None, username=None, password=None,
-                 wsdl_location="local", timeout=30, plugins=[]):
+                 wsdl_location="local", timeout=30, plugins=[], sslcontext=None):
         self._logged_in = False
         if server is None:
             server = _config_value("general", "server")
@@ -77,6 +118,10 @@ class Client(suds.client.Client):
             raise ConfigError("username must be set in config file or Client()")
         if password is None:
             raise ConfigError("password must be set in config file or Client()")
+        if sslcontext is not None:
+            self.transport = HTTPSClientContextTransport(sslcontext)
+        else:
+            self.transport = HttpTransport()
         self.server = server
         self.username = username
         self.password = password
@@ -97,7 +142,7 @@ class Client(suds.client.Client):
         try:
             # Add ExtraConfigPlugin to the plugins
             plugins.append(ExtraConfigPlugin())
-            suds.client.Client.__init__(self, wsdl_uri, plugins=plugins)
+            suds.client.Client.__init__(self, wsdl_uri, plugins=plugins, transport=self.transport)
         except URLError:
             logger.critical("Failed to connect to %s", self.server)
             raise
@@ -108,13 +153,13 @@ class Client(suds.client.Client):
             logger.critical("Failed to load the remote WSDL from %s", wsdl_uri)
             raise
         self.options.transport.options.timeout = timeout
-        self.set_options(location=url)
+        self.set_options(location=url, transport=self.transport)
         mo_ref = soap.ManagedObjectReference("ServiceInstance",
                                              "ServiceInstance")
         self.si = ServiceInstance(mo_ref, self) 
         try:
             self.sc = self.si.RetrieveServiceContent()
-        except URLError, e:
+        except URLError as e:
             logger.critical("Failed to connect to %s" % self.server)
             logger.critical("urllib2 said: %s" % e.reason) 
             raise
@@ -397,7 +442,8 @@ class Client(suds.client.Client):
                       'datacenter_vm_traversal_spec',
                       'compute_resource_rp_traversal_spec',
                       'compute_resource_host_traversal_spec',
-                      'host_vm_traversal_spec']
+                      'host_vm_traversal_spec',
+                      'datacenter_datastore_traversal_spec']
 
         # Create a selection spec for each of the strings specified above
         selection_specs = [
@@ -435,6 +481,12 @@ class Client(suds.client.Client):
         dhts.path = 'hostFolder'
         dhts.selectSet = [selection_specs[2]]
 
+        dsts = self.create('TraversalSpec')
+        dsts.name = 'datacenter_datastore_traversal_spec'
+        dsts.type = 'Datacenter'
+        dsts.path = 'datastoreFolder'
+        dsts.selectSet = [selection_specs[2]]
+
         dvts = self.create('TraversalSpec')
         dvts.name = 'datacenter_vm_traversal_spec'
         dvts.type = 'Datacenter'
@@ -454,12 +506,12 @@ class Client(suds.client.Client):
         fts.selectSet = [selection_specs[2], selection_specs[3],
                           selection_specs[4], selection_specs[5],
                           selection_specs[6], selection_specs[7],
-                          selection_specs[1]]
+                          selection_specs[1], selection_specs[8]]
 
         obj_spec = self.create('ObjectSpec')
         obj_spec.obj = begin_entity
         obj_spec.selectSet = [fts, dvts, dhts, crhts, crrts,
-                               rpts, hvts, rpvts]
+                               rpts, hvts, rpvts, dsts]
 
         pfs = self.create('PropertyFilterSpec')
         pfs.propSet = [property_spec]
@@ -566,7 +618,7 @@ class Client(suds.client.Client):
         property_spec = self.create('PropertySpec')
         property_spec.type = view_type
         property_spec.all = False
-        property_spec.pathSet = filter.keys()
+        property_spec.pathSet = list(filter.keys())
 
         pfs = self.get_search_filter_spec(begin_entity, property_spec)
 
@@ -580,7 +632,7 @@ class Client(suds.client.Client):
             # If no filter is specified we just return the first item
             # in the list of returned objects
             logger.debug("Creating class in find_entity_view (filter)")
-            view = kls(obj_contents[0].obj, self)
+            view = kls(obj_contents[0].obj._mo_ref, self)
             logger.debug("Completed creating class in find_entity_view (filter)")
             #view.update_view_data(properties)
             return view
